using System;
using System.Collections.Generic;
using System.Text;
using System.Query;
using System.Xml.XLinq;

namespace LinqSharp
{
    public static class CustomExtensions
    {
        public static string ChangeCase(this string source)
        {
            return new System.Globalization.CultureInfo("en-US").TextInfo.ToTitleCase(source);
        }

        public static XElement ToXml<T>(this IEnumerable<T> source, XName name)
        {
            return new XElement(name,
                from item in source select new XElement("item", item));
        }

    }

    public static class CustomExtenders2
    {
        public static string ChangeCase(this string source)
        {
            return source.ToString().ToUpper();
        }

    }

    /* Why Extension Methods:
         * Extension Methods are useful because without them we would have to 
         * write code which is more difficult to comprehend.
         * With Extension methods we can write:
         * 
         *      var results = Customers
         *          .Where(c => c.City == "London")
         *          .OrderBy(c => c.CompanyName)
         *          .Select(c => c.CompanyName)
         *          .ToArray
         *          
         * Without extension methods, we would have to write the following:
                 var results3 = System.Query.Sequence.ToArray(
                                          System.Query.Sequence.Select(
                                            System.Query.Sequence.OrderBy(
                                                System.Query.Sequence.Where(db.Customers
                                                , c => c.City == "London")
                                            , c => c.CompanyName)
                                          , c => c.CompanyName)
                                        );
                */

    //public static class Sequence
    // See sequence.cs in the CTP documentation folder
    //{
        
    //    public static IEnumerable<T> Where1<T>(this IEnumerable<T> source, Func<T, bool> predicate) {
    //        if (source == null) throw Error.ArgumentNull("source");
    //        if (predicate == null) throw Error.ArgumentNull("predicate");
    //        return WhereIterator<T>(source, predicate);
    //    }

    //    static IEnumerable<T> WhereIterator1<T>(IEnumerable<T> source, Func<T, bool> predicate) {
    //        foreach (T element in source) {
    //            if (predicate(element)) yield return element;
    //        }
    //    }
    //}
}
