﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Query;
using System.Xml.XLinq;
using System.Data.DLinq;
using System.IO;
using DemoData;
using System.Expressions;

namespace LinqSharp
{
    class Program
    {
        static void Main(string[] args)
        {
            //start:
            var results = from f in new DirectoryInfo(@"C:\windows").GetFiles()
                          where f.extension == ".exe" 
                          select new{f.Name, f.LastAccessTime};
            //becomes:
            //Func<FileInfo,bool> filter=f => f.Extension == ".exe";
            //Func<FileInfo,object> selector=f => new {f.Name, f.LastAccessTime};
            //var results = new DirectoryInfo(@"C:\windows").GetFiles()
            //    .Where(filter)
            //    .Select(selector);

            //Dumper.Write(results);

            //LambdaDictionaryDemo(); 

            //Extension method demo
            //Console.WriteLine("This is  a test".ChangeCase());
            //Console.WriteLine(CustomExtensions.ChangeCase("This is a test"));

            //ExpressionTreeDemo();
            //XLinqDemo();
     
            Console.ReadKey();
        }

        static void LambdaDictionaryDemo()
        {
            Func<FileInfo, object> selector = f => new { f.Name, f.LastAccessTime };
            Dictionary<string, Func<FileInfo, bool>> tests = new Dictionary<string, Func<FileInfo, bool>>();
            tests.Add("Executables", f => f.Extension == ".exe");
            tests.Add("Text", f => f.Extension == ".log" | f.Extension == ".txt");
            tests.Add("Recent", f => f.LastAccessTime.CompareTo(System.DateTime.Now.AddDays(-7)) > 0);
            tests.Add("KB", f => System.Text.RegularExpressions.Regex.IsMatch(f.Name, "KB[0-9]{6}.log"));
            while (true)
            {
                Console.Write("Run which test? ");
                string answer = Console.ReadLine();
                var results = new DirectoryInfo(@"C:\windows").GetFiles()
                    .Where(tests[answer])
                    .Select(selector);
                Dumper.Write(results);
            }
        }

        static void ExpressionTreeDemo()
        {
            Func<int, int> add = x => x + 1;
            System.Expressions.Expression<Func<int, int>> AddExpression = x => x + 1;
            BinaryExpression addBody = (BinaryExpression)AddExpression.Body;
            Console.WriteLine("Body={0}", addBody);
            Console.WriteLine("Right = {0}", addBody.Right);
            Console.WriteLine("Left  = (0)", addBody.Left);
            Console.WriteLine("NodeType = {0}", addBody.NodeType);
        }

        static void XLinqDemo()
        {
            //Uses Functional Construction
            XElement x = new XElement("LinqEssentials",
                       new XElement("Category", "LINQ"),
                       new XElement("Category", "DLINQ"),
                       new XElement("Category", "XLINQ"));
            XElement y = new XElement("LanguageFeatures");
            y.Add(new XElement("release", new XAttribute("version", "2.0"),
                new XElement("feature", "Generics"),
                new XElement("feature", "Iterators"),
                new XElement("feature", "Anonymous Delegates"),
                new XElement("feature", "Nullable Types")
                ));
            y.Add(new XElement("release", new XAttribute("version", "3.0"),
                 new XElement("feature", "Implicit Types"),
                 new XElement("feature", "Extension methods"),
                 new XElement("feature", "Lambda Expressions"),
                 new XElement("feature", "Object initializers"),
                 new XElement("feature", "Anonymous Types"),
                 new XElement("feature", "Expression Trees")
                 ));
            y.Add(new XElement("release", new XAttribute("version", "9.0"),
                new XElement("feature", "dynamic interfaces"),
                new XElement("feature", "dynamic identifiers"),
                new XElement("feature", "XML Literals")
                ));
            XElement z = new XElement("LinqRocksBecause");
            z.Add(x);
            z.Add(y);

            Console.BackgroundColor = ConsoleColor.Cyan;
            Console.ForegroundColor = ConsoleColor.Blue;
            //Write the original out
            Console.WriteLine(z);
            System.Diagnostics.Debug.Write(z);
            //save it to disk
            z.Save("c:\\LinqRocks.xml");

            //reload from disk
            Console.BackgroundColor = ConsoleColor.White;
            Console.WriteLine("Reloading");
            XDocument reload = XDocument.Load("c:\\linqrocks.xml");

            //output it again
            Console.ForegroundColor = ConsoleColor.Black;
            Console.Write(reload);

            //querying
            Console.BackgroundColor = ConsoleColor.Yellow;
            foreach (XElement c in z.Descendants("feature"))
            {
                Console.WriteLine(c);
            }

            Console.ReadKey();
        }
    }
}
