using System;
using System.Collections.Generic;
using System.Text;

namespace LinqSharp
{

    static class ScriptedDemos
    {

        static void ExpressionTreeDemo()
        {
            Func<int, int> add = x => x + 1;
            System.Expressions.Expression<Func<int, int>> AddExpression = x => x + 1;
            BinaryExpression addBody = (BinaryExpression)AddExpression.Body;
            Console.WriteLine("Body={0}", addBody);
            Console.WriteLine("Right = {0}", addBody.Right);
            Console.WriteLine("Left  = (0)", addBody.Left);
            Console.WriteLine("NodeType = {0}", addBody.NodeType);
        }

        static void LambdaDictionaryDemo()
        {
            Func<string, object> selector = f => new { FileName = f, LastAccessed = File.GetLastAccessTime(f) };
            Dictionary<string, Func<string, bool>> tests = new Dictionary<string, Func<string, bool>>();
            tests.Add("Executables", f => Path.GetExtension(f) == ".exe");
            tests.Add("Recent", f => File.GetLastAccessTime(f).CompareTo(System.DateTime.Now.AddDays(-7)) > 0);
            tests.Add("KB", f => System.Text.RegularExpressions.Regex.IsMatch(f, "KB[0-9]{6}.log"));
            while (true)
            {
                Console.Write("Run which test? ");
                string answer = Console.ReadLine();
                var results = Directory.GetFiles("C:\\Windows")
                    .Where(tests[answer])
                    .Select(selector);
                ObjectDumper.Write(results);
            }
        }

        static void XLinqDemo()
        {
            //Uses Functional Construction
            XElement x = new XElement("LinqEssentials",
                       new XElement("Category", "LINQ"),
                       new XElement("Category", "DLINQ"),
                       new XElement("Category", "XLINQ"));
            XElement y = new XElement("LanguageFeatures");
            y.Add(new XElement("release", new XAttribute("version", "2.0"),
                new XElement("feature", "Generics"),
                new XElement("feature", "Iterators"),
                new XElement("feature", "Anonymous Delegates"),
                new XElement("feature", "Nullable Types")
                ));
            y.Add(new XElement("release", new XAttribute("version", "3.0"),
                 new XElement("feature", "Implicit Types"),
                 new XElement("feature", "Extension methods"),
                 new XElement("feature", "Lambda Expressions"),
                 new XElement("feature", "Object initializers"),
                 new XElement("feature", "Anonymous Types"),
                 new XElement("feature", "Expression Trees")
                 ));
            y.Add(new XElement("release", new XAttribute("version", "9.0"),
                new XElement("feature", "dynamic interfaces"),
                new XElement("feature", "dynamic identifiers"),
                new XElement("feature", "XML Literals")
                ));
            XElement z = new XElement("LinqRocksBecause");
            z.Add(x);
            z.Add(y);

            Console.BackgroundColor = ConsoleColor.Cyan;
            Console.ForegroundColor = ConsoleColor.Blue;
            Console.WriteLine(z);
            System.Diagnostics.Debug.Write(z);

            z.Save("c:\\LinqRocks.xml");

            Console.BackgroundColor = ConsoleColor.White;
            Console.WriteLine("Reloading");
            XDocument reload = XDocument.Load("c:\\linqrocks.xml");

            Console.ForegroundColor = ConsoleColor.Black;
            Console.Write(reload);


            Console.BackgroundColor = ConsoleColor.Yellow;
            foreach (XElement c in z.Descendants("feature"))
            {
                Console.WriteLine(c);
            }

            Console.ReadKey();
        }
    }
}
